@@ -9,6 +9,8 @@ from drf_spectacular.plumbing import (
 from drf_spectacular.types import OpenApiTypes
 from drf_spectacular.utils import OpenApiParameter
 
+_NoHint = object()
+
 
 class DjangoFilterExtension(OpenApiFilterExtension):
     """
@@ -31,7 +33,9 @@ class DjangoFilterExtension(OpenApiFilterExtension):
     - ``TypedMultipleChoiceFilter``: enum, multi handled
 
     In case of warnings or incorrect filter types, you can manually override the underlying
-    field type with a manual ``extend_schema_field`` decoration.
+    field type with a manual ``extend_schema_field`` decoration. Alternatively, if you have a
+    filter method for your filter field, you can attach ``extend_schema_field`` to that filter
+    method.
 
     .. code-block::
 
@@ -78,12 +82,24 @@ class DjangoFilterExtension(OpenApiFilterExtension):
             filters.IsoDateTimeFromToRangeFilter: OpenApiTypes.DATETIME,
             filters.DateTimeFromToRangeFilter: OpenApiTypes.DATETIME,
         }
-        if has_override(filter_field, 'field'):
-            annotation = get_override(filter_field, 'field')
+        filter_method = self._get_filter_method(filterset_class, filter_field)
+        filter_method_hint = self._get_filter_method_hint(filter_method)
+        filter_choices = self._get_explicit_filter_choices(filter_field)
+
+        if has_override(filter_field, 'field') or has_override(filter_method, 'field'):
+            annotation = (
+                get_override(filter_field, 'field') or get_override(filter_method, 'field')
+            )
             if is_basic_type(annotation):
                 schema = build_basic_type(annotation)
             else:
+                # allow injecting raw schema via @extend_schema_field decorator
                 schema = annotation
+        elif filter_method_hint is not _NoHint:
+            if is_basic_type(filter_method_hint):
+                schema = build_basic_type(filter_method_hint)
+            else:
+                schema = build_basic_type(OpenApiTypes.STR)
         elif isinstance(filter_field, tuple(unambiguous_mapping)):
             for cls in filter_field.__class__.__mro__:
                 if cls in unambiguous_mapping:
@@ -92,35 +108,34 @@ class DjangoFilterExtension(OpenApiFilterExtension):
         elif isinstance(filter_field, (filters.NumberFilter, filters.NumericRangeFilter)):
             # NumberField is underspecified by itself. try to find the
             # type that makes the most sense or default to generic NUMBER
-            if filter_field.method:
-                schema = self._build_filter_method_type(filterset_class, filter_field)
-                if schema['type'] not in ['integer', 'number']:
-                    schema = build_basic_type(OpenApiTypes.NUMBER)
+            model_field = self._get_model_field(filter_field, model)
+            if isinstance(model_field, (models.IntegerField, models.AutoField)):
+                schema = build_basic_type(OpenApiTypes.INT)
+            elif isinstance(model_field, models.FloatField):
+                schema = build_basic_type(OpenApiTypes.FLOAT)
+            elif isinstance(model_field, models.DecimalField):
+                schema = build_basic_type(OpenApiTypes.NUMBER)  # TODO may be improved
             else:
-                model_field = self._get_model_field(filter_field, model)
-                if isinstance(model_field, (models.IntegerField, models.AutoField)):
-                    schema = build_basic_type(OpenApiTypes.INT)
-                elif isinstance(model_field, models.FloatField):
-                    schema = build_basic_type(OpenApiTypes.FLOAT)
-                elif isinstance(model_field, models.DecimalField):
-                    schema = build_basic_type(OpenApiTypes.NUMBER)  # TODO may be improved
-                else:
-                    schema = build_basic_type(OpenApiTypes.NUMBER)
-        elif filter_field.method:
-            # try to make best effort on the given method
-            schema = self._build_filter_method_type(filterset_class, filter_field)
-        else:
+                schema = build_basic_type(OpenApiTypes.NUMBER)
+        elif isinstance(filter_field, filters.ChoiceFilter):
             try:
-                # the last resort is to lookup the type via the model or queryset field.
-                # first search for the field in the model as this has the least amount of
-                # potential side effects. Only after that fails, attempt to call
-                # get_queryset() to check for potential query annotations.
-                model_field = self._get_model_field(filter_field, model)
-                if not isinstance(model_field, models.Field):
-                    qs = auto_schema.view.get_queryset()
-                    model_field = qs.query.annotations[filter_field.field_name].field
-                schema = auto_schema._map_model_field(model_field, direction=None)
-            except Exception as exc:
+                schema = self._get_schema_from_model_field(auto_schema, filter_field, model)
+            except Exception:
+                if filter_choices and is_basic_type(type(filter_choices[0])):
+                    # fallback to type guessing from first choice element
+                    schema = build_basic_type(type(filter_choices[0]))
+                else:
+                    warn(
+                        f'Unable to guess choice types from values, filter method\'s type hint '
+                        f'or find "{field_name}" in model. Defaulting to string.'
+                    )
+                    schema = build_basic_type(OpenApiTypes.STR)
+        else:
+            # the last resort is to look up the type via the model or queryset field
+            # and emit a warning if we were unsuccessful.
+            try:
+                schema = self._get_schema_from_model_field(auto_schema, filter_field, model)
+            except Exception as exc:  # pragma: no cover
                 warn(
                     f'Exception raised while trying resolve model field for django-filter '
                     f'field "{field_name}". Defaulting to string (Exception: {exc})'
@@ -132,8 +147,9 @@ class DjangoFilterExtension(OpenApiFilterExtension):
         schema.pop('readOnly', None)
         # enrich schema with additional info from filter_field
         enum = schema.pop('enum', None)
-        if 'choices' in filter_field.extra:
-            enum = [c for c, _ in filter_field.extra['choices']]
+        # explicit filter choices may disable enum retrieved from model
+        if filter_choices is not None:
+            enum = filter_choices
         if enum:
             schema['enum'] = sorted(enum, key=str)
 
@@ -182,22 +198,43 @@ class DjangoFilterExtension(OpenApiFilterExtension):
             for field_name in field_names
         ]
 
-    def _build_filter_method_type(self, filterset_class, filter_field):
+    def _get_filter_method(self, filterset_class, filter_field):
         if callable(filter_field.method):
-            filter_method = filter_field.method
+            return filter_field.method
+        elif isinstance(filter_field.method, str):
+            return getattr(filterset_class, filter_field.method)
         else:
-            filter_method = getattr(filterset_class, filter_field.method)
+            return None
 
+    def _get_filter_method_hint(self, filter_method):
         try:
-            filter_method_hints = get_type_hints(filter_method)
+            return get_type_hints(filter_method)['value']
         except:  # noqa: E722
-            filter_method_hints = {}
+            return _NoHint
 
-        if 'value' in filter_method_hints and is_basic_type(filter_method_hints['value']):
-            return build_basic_type(filter_method_hints['value'])
+    def _get_explicit_filter_choices(self, filter_field):
+        if 'choices' not in filter_field.extra:
+            return None
+        elif callable(filter_field.extra['choices']):
+            # choices function may utilize the DB, so refrain from actually calling it.
+            return []
         else:
-            return build_basic_type(OpenApiTypes.STR)
+            return [c for c, _ in filter_field.extra['choices']]
 
     def _get_model_field(self, filter_field, model):
+        if not filter_field.field_name:
+            return None
         path = filter_field.field_name.split('__')
         return follow_field_source(model, path, emit_warnings=False)
+
+    def _get_schema_from_model_field(self, auto_schema, filter_field, model):
+        # Has potential to throw exceptions. Needs to be wrapped in try/except!
+        #
+        # first search for the field in the model as this has the least amount of
+        # potential side effects. Only after that fails, attempt to call
+        # get_queryset() to check for potential query annotations.
+        model_field = self._get_model_field(filter_field, model)
+        if not isinstance(model_field, models.Field):
+            qs = auto_schema.view.get_queryset()
+            model_field = qs.query.annotations[filter_field.field_name].field
+        return auto_schema._map_model_field(model_field, direction=None)
