@@ -4,6 +4,15 @@ import sys
 from collections import defaultdict
 from typing import DefaultDict
 
+if sys.version_info >= (3, 8):
+    from typing import (  # type: ignore[attr-defined] # noqa: F401
+        Final, Literal, TypedDict, _TypedDictMeta,
+    )
+else:
+    from typing_extensions import (  # type: ignore[attr-defined] # noqa: F401
+        Final, Literal, TypedDict, _TypedDictMeta,
+    )
+
 
 class GeneratorStats:
     _warn_cache: DefaultDict[str, int] = defaultdict(int)
@@ -51,12 +60,22 @@ class GeneratorStats:
 GENERATOR_STATS = GeneratorStats()
 
 
-def warn(msg):
-    GENERATOR_STATS.emit(msg, 'warning')
+def warn(msg, delayed=None):
+    if delayed:
+        warnings = get_override(delayed, 'warnings', [])
+        warnings.append(msg)
+        set_override(delayed, 'warnings', warnings)
+    else:
+        GENERATOR_STATS.emit(msg, 'warning')
 
 
-def error(msg):
-    GENERATOR_STATS.emit(msg, 'error')
+def error(msg, delayed=None):
+    if delayed:
+        errors = get_override(delayed, 'errors', [])
+        errors.append(msg)
+        set_override(delayed, 'errors', errors)
+    else:
+        GENERATOR_STATS.emit(msg, 'error')
 
 
 def reset_generator_stats():
@@ -101,6 +120,8 @@ def get_override(obj, prop, default=None):
 def set_override(obj, prop, value):
     if not hasattr(obj, '_spectacular_annotation'):
         obj._spectacular_annotation = {}
+    elif '_spectacular_annotation' not in obj.__dict__:
+        obj._spectacular_annotation = obj._spectacular_annotation.copy()
     obj._spectacular_annotation[prop] = value
     return obj
 
