@@ -27,16 +27,17 @@ from drf_spectacular.extensions import (
 from drf_spectacular.plumbing import (
     ComponentRegistry, ResolvedComponent, UnableToProceedError, append_meta,
     assert_basic_serializer, build_array_type, build_basic_type, build_choice_field,
-    build_examples_list, build_generic_type, build_media_type_object, build_object_type,
-    build_parameter_type, error, follow_field_source, follow_model_field_lookup, force_instance,
-    get_doc, get_type_hints, get_view_model, is_basic_serializer, is_basic_type, is_field,
-    is_list_serializer, is_patched_serializer, is_serializer, is_trivial_string_variation,
+    build_examples_list, build_generic_type, build_listed_example_value, build_media_type_object,
+    build_mocked_view, build_object_type, build_parameter_type, error, follow_field_source,
+    follow_model_field_lookup, force_instance, get_doc, get_type_hints, get_view_model,
+    is_basic_serializer, is_basic_type, is_field, is_list_serializer, is_patched_serializer,
+    is_serializer, is_trivial_string_variation, modify_media_types_for_versioning,
     resolve_django_path_parameter, resolve_regex_path_parameter, resolve_type_hint, safe_ref,
-    sanitize_specification_extensions, warn,
+    sanitize_specification_extensions, warn, whitelisted,
 )
 from drf_spectacular.settings import spectacular_settings
 from drf_spectacular.types import OpenApiTypes
-from drf_spectacular.utils import OpenApiParameter, OpenApiResponse
+from drf_spectacular.utils import OpenApiCallback, OpenApiParameter, OpenApiResponse
 
 
 class AutoSchema(ViewInspector):
@@ -65,6 +66,10 @@ class AutoSchema(ViewInspector):
         if summary:
             operation['summary'] = summary
 
+        external_docs = self._get_external_docs()
+        if external_docs:
+            operation['externalDocs'] = external_docs
+
         parameters = self._get_parameters()
         if parameters:
             operation['parameters'] = parameters
@@ -91,6 +96,10 @@ class AutoSchema(ViewInspector):
         if extensions:
             operation.update(sanitize_specification_extensions(extensions))
 
+        callbacks = self._get_callbacks()
+        if callbacks:
+            operation['callbacks'] = callbacks
+
         return operation
 
     def _is_list_view(self, serializer=None):
@@ -148,10 +157,22 @@ class AutoSchema(ViewInspector):
 
                 if is_basic_type(parameter.type):
                     schema = build_basic_type(parameter.type)
-                elif is_serializer(parameter.type):
+                elif is_basic_serializer(parameter.type):
                     schema = self.resolve_serializer(parameter.type, 'request').ref
-                else:
+                elif isinstance(parameter.type, dict):
                     schema = parameter.type
+                else:
+                    warn(f'unsupported type for parameter "{parameter.name}". Skipping.')
+                    continue
+
+                if parameter.many:
+                    if is_basic_type(parameter.type):
+                        schema = build_array_type(schema)
+                    else:
+                        warn(
+                            f'parameter "{parameter.name}" has many=True and is not a basic type. '
+                            f'many=True only makes sense for basic types. Ignoring.'
+                        )
 
                 if parameter.exclude:
                     result[parameter.name, parameter.location] = None
@@ -167,19 +188,23 @@ class AutoSchema(ViewInspector):
                         style=parameter.style,
                         explode=parameter.explode,
                         default=parameter.default,
+                        allow_blank=parameter.allow_blank,
                         examples=build_examples_list(parameter.examples),
                         extensions=parameter.extensions,
                     )
             elif is_basic_serializer(parameter):
                 # explode serializer into separate parameters. defaults to QUERY location
+                parameter = force_instance(parameter)
                 mapped = self._map_serializer(parameter, 'request')
                 for property_name, property_schema in mapped['properties'].items():
+                    field = parameter.fields.get(property_name)
                     result[property_name, OpenApiParameter.QUERY] = build_parameter_type(
                         name=property_name,
                         schema=property_schema,
                         description=property_schema.pop('description', None),
                         location=OpenApiParameter.QUERY,
-                        required=property_name in mapped.get('required', []),
+                        allow_blank=getattr(field, 'allow_blank', True),
+                        required=field.required,
                     )
             else:
                 warn(f'could not resolve parameter annotation {parameter}. Skipping.')
@@ -250,6 +275,17 @@ class AutoSchema(ViewInspector):
         """ override this for custom behaviour """
         return None
 
+    def _get_external_docs(self):
+        external_docs = self.get_external_docs()
+        if isinstance(external_docs, str):
+            return {'url': external_docs}
+        else:
+            return external_docs
+
+    def get_external_docs(self):
+        """ override this for custom behaviour """
+        return None
+
     def get_auth(self):
         """
         Obtains authentication classes and permissions from view. If authentication
@@ -260,10 +296,7 @@ class AutoSchema(ViewInspector):
         auths = []
 
         for authenticator in self.view.get_authenticators():
-            if (
-                spectacular_settings.AUTHENTICATION_WHITELIST
-                and authenticator.__class__ not in spectacular_settings.AUTHENTICATION_WHITELIST
-            ):
+            if not whitelisted(authenticator, spectacular_settings.AUTHENTICATION_WHITELIST, True):
                 continue
 
             scheme = OpenApiAuthenticationExtension.get_match(authenticator)
@@ -321,6 +354,68 @@ class AutoSchema(ViewInspector):
     def get_extensions(self) -> typing.Dict[str, typing.Any]:
         return {}
 
+    def _get_callbacks(self):
+        """
+        Creates a mocked view for every callback. The given extend_schema decorator then
+        specifies the expectations on the receiving end of the callback. Effectively
+        simulates a sub-schema from the opposing perspective via a virtual view definition.
+        """
+        result = {}
+
+        for callback in self.get_callbacks():
+            if isinstance(callback.decorator, dict):
+                methods = callback.decorator
+            else:
+                methods = {'post': callback.decorator}
+
+            path_items = {}
+
+            for method, decorator in methods.items():
+                # a dict indicates a raw schema; use directly
+                if isinstance(decorator, dict):
+                    path_items[method.lower()] = decorator
+                    continue
+
+                mocked_view = build_mocked_view(
+                    method=method,
+                    path=callback.path,
+                    extend_schema_decorator=decorator,
+                    registry=self.registry
+                )
+                operation = {}
+
+                description = mocked_view.schema.get_description()
+                if description:
+                    operation['description'] = description
+
+                summary = mocked_view.schema.get_summary()
+                if summary:
+                    operation['summary'] = summary
+
+                request_body = mocked_view.schema._get_request_body()
+                if request_body:
+                    operation['requestBody'] = request_body
+
+                deprecated = mocked_view.schema.is_deprecated()
+                if deprecated:
+                    operation['deprecated'] = deprecated
+
+                operation['responses'] = mocked_view.schema._get_response_bodies()
+
+                extensions = mocked_view.schema.get_extensions()
+                if extensions:
+                    operation.update(sanitize_specification_extensions(extensions))
+
+                path_items[method.lower()] = operation
+
+            result[callback.name] = {callback.path: path_items}
+
+        return result
+
+    def get_callbacks(self) -> typing.List[OpenApiCallback]:
+        """ override this for custom behaviour """
+        return []
+
     def get_operation_id(self):
         """ override this for custom behaviour """
         tokenized_path = self._tokenize_path()
@@ -410,14 +505,15 @@ class AutoSchema(ViewInspector):
 
         return parameters
 
-    def _get_filter_parameters(self):
+    def get_filter_backends(self):
+        """ override this for custom behaviour """
         if not self._is_list_view():
             return []
-        if getattr(self.view, 'filter_backends', None) is None:
-            return []
+        return getattr(self.view, 'filter_backends', [])
 
+    def _get_filter_parameters(self):
         parameters = []
-        for filter_backend in self.view.filter_backends:
+        for filter_backend in self.get_filter_backends():
             filter_extension = OpenApiFilterExtension.get_match(filter_backend())
             if filter_extension:
                 parameters += filter_extension.get_schema_operation_parameters(self)
@@ -497,7 +593,7 @@ class AutoSchema(ViewInspector):
             return build_basic_type(OpenApiTypes.STR)
 
     def _map_serializer_field(self, field, direction, bypass_extensions=False):
-        meta = self._get_serializer_field_meta(field)
+        meta = self._get_serializer_field_meta(field, direction)
 
         if has_override(field, 'field'):
             override = get_override(field, 'field')
@@ -654,7 +750,7 @@ class AutoSchema(ViewInspector):
                 content = {**build_basic_type(OpenApiTypes.STR), 'format': 'decimal'}
                 if field.max_whole_digits:
                     content['pattern'] = (
-                        fr'^\d{{0,{field.max_whole_digits}}}'
+                        fr'^-?\d{{0,{field.max_whole_digits}}}'
                         fr'(?:\.\d{{0,{field.decimal_places}}})?$'
                     )
             else:
@@ -671,7 +767,7 @@ class AutoSchema(ViewInspector):
             return append_meta(content, meta)
 
         if isinstance(field, serializers.FloatField):
-            content = build_basic_type(OpenApiTypes.FLOAT)
+            content = build_basic_type(OpenApiTypes.DOUBLE)
             self._insert_min_max(field, content)
             return append_meta(content, meta)
 
@@ -692,7 +788,14 @@ class AutoSchema(ViewInspector):
             return append_meta(content, meta)
 
         if isinstance(field, serializers.SerializerMethodField):
-            method = getattr(field.parent, field.method_name)
+            method = getattr(field.parent, field.method_name, None)
+            if method is None:
+                error(
+                    f'SerializerMethodField "{field.field_name}" is missing required method '
+                    f'"{field.method_name}". defaulting to "string".'
+                )
+                return append_meta(build_basic_type(OpenApiTypes.STR), meta)
+
             return append_meta(self._map_response_type_hint(method), meta)
 
         if isinstance(field, (serializers.BooleanField, serializers.NullBooleanField)):
@@ -798,7 +901,7 @@ class AutoSchema(ViewInspector):
             del schema['required']
         return schema
 
-    def _get_serializer_field_meta(self, field):
+    def _get_serializer_field_meta(self, field, direction):
         if not isinstance(field, serializers.Field):
             return {}
 
@@ -809,6 +912,14 @@ class AutoSchema(ViewInspector):
             meta['writeOnly'] = True
         if field.allow_null:
             meta['nullable'] = True
+        if isinstance(field, serializers.CharField) and not field.allow_blank:
+            # blank check only applies to inbound requests
+            if spectacular_settings.COMPONENT_SPLIT_REQUEST:
+                if direction == 'request':
+                    meta['minLength'] = 1
+            elif spectacular_settings.ENFORCE_NON_BLANK_FIELDS:
+                if not field.read_only:
+                    meta['minLength'] = 1
         if field.default is not None and field.default != empty and not callable(field.default):
             if isinstance(field, (serializers.ModelField, serializers.SerializerMethodField)):
                 # Skip coercion for lack of a better solution. ModelField.to_representation()
@@ -950,14 +1061,25 @@ class AutoSchema(ViewInspector):
         return f'Paginated{serializer_name}List'
 
     def map_parsers(self):
-        return list(dict.fromkeys([p.media_type for p in self.view.get_parsers()]))
+        return list(dict.fromkeys([
+            p.media_type for p in self.view.get_parsers()
+            if whitelisted(p, spectacular_settings.PARSER_WHITELIST)
+        ]))
 
     def map_renderers(self, attribute):
         assert attribute in ['media_type', 'format']
+
+        # Either use whitelist or default back to old behavior by excluding BrowsableAPIRenderer
+        def use_renderer(r):
+            if spectacular_settings.RENDERER_WHITELIST:
+                return whitelisted(r, spectacular_settings.RENDERER_WHITELIST)
+            else:
+                return not isinstance(r, renderers.BrowsableAPIRenderer)
+
         return list(dict.fromkeys([
             getattr(r, attribute).split(';')[0]
             for r in self.view.get_renderers()
-            if not isinstance(r, renderers.BrowsableAPIRenderer) and getattr(r, attribute, None)
+            if use_renderer(r) and hasattr(r, attribute)
         ]))
 
     def _get_serializer(self):
@@ -994,6 +1116,7 @@ class AutoSchema(ViewInspector):
             )
 
     def get_examples(self):
+        """ override this for custom behaviour """
         return []
 
     def _get_examples(self, serializer, direction, media_type, status_code=None, extras=None):
@@ -1002,6 +1125,8 @@ class AutoSchema(ViewInspector):
         # don't let the parameter examples influence the serializer example retrieval
         examples = [e for e in self.get_examples() if not e.parameter_only]
 
+        # Examples from Serializers via @extend_schema_serializer are only considered, if
+        # there were no higher priority examples directly from view annotation.
         if not examples:
             if is_list_serializer(serializer):
                 examples = get_override(serializer.child, 'examples', [])
@@ -1021,6 +1146,15 @@ class AutoSchema(ViewInspector):
                 continue
             if status_code and status_code not in example.status_codes:
                 continue
+
+            if (
+                self._is_list_view(serializer)
+                and get_override(serializer, 'many') is not False
+            ):
+                # contain modification to this context so "listing" is not propagated elsewhere
+                example = copy.copy(example)
+                example.value = build_listed_example_value(example.value, self._get_paginator(), direction)
+
             filtered_examples.append(example)
 
         return build_examples_list(filtered_examples)
@@ -1218,6 +1352,8 @@ class AutoSchema(ViewInspector):
         if not media_types:
             media_types = self.map_renderers('media_type')
 
+        media_types = modify_media_types_for_versioning(self.view, media_types)
+
         return {
             **headers,
             'content': {
@@ -1264,6 +1400,7 @@ class AutoSchema(ViewInspector):
                 style=parameter.style,
                 explode=parameter.explode,
                 default=parameter.default,
+                allow_blank=parameter.allow_blank,
                 examples=build_examples_list(parameter.examples),
                 extensions=parameter.extensions,
             )
@@ -1299,6 +1436,13 @@ class AutoSchema(ViewInspector):
         if direction == 'request' and spectacular_settings.COMPONENT_SPLIT_REQUEST:
             name = name + 'Request'
 
+        if not re.match(r'^[\w.-]+$', name):
+            warn(
+                f'Component name "{name}" contains illegal characters. Only "A-Z a-z 0-9 - . _" '
+                f'are allowed. Furthermore, "-" and "." are discoursed due to potential tooling '
+                f'issues. This likely leads to an invalid schema.'
+            )
+
         return name
 
     def resolve_serializer(self, serializer, direction) -> ResolvedComponent:
