@@ -2,6 +2,12 @@ import inspect
 import sys
 from typing import Any, Callable, Dict, List, Optional, Tuple, Type, TypeVar, Union
 
+# direct import due to https://github.com/microsoft/pyright/issues/3025
+if sys.version_info >= (3, 8):
+    from typing import Final, Literal
+else:
+    from typing_extensions import Final, Literal
+
 from rest_framework.fields import Field, empty
 from rest_framework.serializers import Serializer
 from rest_framework.settings import api_settings
@@ -11,16 +17,10 @@ from drf_spectacular.drainage import (
 )
 from drf_spectacular.types import OpenApiTypes, _KnownPythonTypes
 
-if sys.version_info >= (3, 8):
-    from typing import Final, Literal
-else:
-    from typing_extensions import Final, Literal
-
-
 _SerializerType = Union[Serializer, Type[Serializer]]
 _FieldType = Union[Field, Type[Field]]
-
 _ParameterLocationType = Literal['query', 'path', 'header', 'cookie']
+Direction = Literal['request', 'response']
 
 
 class PolymorphicProxySerializer(Serializer):
@@ -44,19 +44,24 @@ class PolymorphicProxySerializer(Serializer):
             return Response(...)
 
     **Beware** that this is not a real serializer and it will raise an AssertionError
-    if used in that way. It **cannot** be used in views as `serializer_class`
+    if used in that way. It **cannot** be used in views as ``serializer_class``
     or as field in an actual serializer. It is solely meant for annotation purposes.
 
     Also make sure that each sub-serializer has a field named after the value of
     ``resource_type_field_name`` (discriminator field). Generated clients will likely
-    depend on the existence of this field. Setting ``resource_type_field_name``
-    to ``None`` will remove the discriminator altogether. This may be useful in
-    certain situations, but will most likely break client generation.
+    depend on the existence of this field.
+
+    Setting ``resource_type_field_name`` to ``None`` will remove the discriminator
+    altogether. This may be useful in certain situations, but will most likely break
+    client generation. Another use-case is explicit control over sub-serializer's ``many``
+    attribute. To explicitly control this aspect, you need disable the discriminator with
+    ``resource_type_field_name=None`` as well as disable automatic list handling with
+    ``many=False``.
 
     It is **strongly** recommended to pass the ``Serializers`` as **list**,
     and by that let *drf-spectacular* retrieve the field and handle the mapping
     automatically. In special circumstances, the field may not available when
-    drf-spectacular processes the serializer. In those cases you can explicitly state
+    *drf-spectacular* processes the serializer. In those cases you can explicitly state
     the mapping with ``{'legal': LegalPersonSerializer, ...}``, but it is then your
     responsibility to have a valid mapping.
     """
@@ -65,12 +70,23 @@ class PolymorphicProxySerializer(Serializer):
             component_name: str,
             serializers: Union[List[_SerializerType], Dict[str, _SerializerType]],
             resource_type_field_name: Optional[str],
-            many: bool = False,
+            many: Optional[bool] = None,
     ):
         self.component_name = component_name
         self.serializers = serializers
         self.resource_type_field_name = resource_type_field_name
-        super().__init__(many=many)
+        if self._many is False:  # type: ignore[attr-defined]
+            set_override(self, 'many', False)
+        super().__init__()
+
+    def __new__(cls, *args, **kwargs):
+        many = kwargs.pop('many', None)
+        if many is True:
+            instance = cls.many_init(*args, **kwargs)
+        else:
+            instance = super().__new__(cls, *args, **kwargs)
+        instance._many = many
+        return instance
 
     @property
     def data(self):
@@ -155,6 +171,8 @@ class OpenApiParameter(OpenApiSchemaBase):
             style: Optional[str] = None,
             explode: Optional[bool] = None,
             default: Any = None,
+            allow_blank: bool = True,
+            many: Optional[bool] = None,
             examples: Optional[List[OpenApiExample]] = None,
             extensions: Optional[Dict[str, Any]] = None,
             exclude: bool = False,
@@ -170,6 +188,8 @@ class OpenApiParameter(OpenApiSchemaBase):
         self.style = style
         self.explode = explode
         self.default = default
+        self.allow_blank = allow_blank
+        self.many = many
         self.examples = examples or []
         self.extensions = extensions
         self.exclude = exclude
@@ -188,7 +208,7 @@ class OpenApiResponse(OpenApiSchemaBase):
     def __init__(
             self,
             response: Any = None,
-            description: Optional[str] = None,
+            description: str = '',
             examples: Optional[List[OpenApiExample]] = None
     ):
         self.response = response
@@ -199,6 +219,36 @@ class OpenApiResponse(OpenApiSchemaBase):
 F = TypeVar('F', bound=Callable[..., Any])
 
 
+class OpenApiCallback(OpenApiSchemaBase):
+    """
+    Helper class to bundle a callback definition. This specifies a view on the callee's
+    side, effectively stating the expectations on the receiving end. Please note that this
+    particular :func:`@extend_schema <.extend_schema>` instance operates from the perspective
+    of the callback origin, which means that ``request`` specifies the outgoing request.
+
+    For convenience sake, we assume the callback sends ``application/json`` and return a ``200``.
+    If that is not sufficient, you can use ``request`` and ``responses`` overloads just as you
+    normally would.
+
+    :param name: Name under which the this callback is listed in the schema.
+    :param path: Path on which the callback operation is performed. To reference request
+        body contents, please refer to OpenAPI specification's
+        `key expressions <https://swagger.io/specification/#key-expression>`_ for valid choices.
+    :param decorator: :func:`@extend_schema <.extend_schema>` decorator that specifies the receiving
+        endpoint. In this special context the allowed parameters are ``requests``, ``responses``,
+        ``summary``, ``description``, ``deprecated``.
+    """
+    def __init__(
+            self,
+            name: str,
+            path: str,
+            decorator: Union[Callable[[F], F], Dict[str, Callable[[F], F]], Dict[str, Any]],
+    ):
+        self.name = name
+        self.path = path
+        self.decorator = decorator
+
+
 def extend_schema(
         operation_id: Optional[str] = None,
         parameters: Optional[List[Union[OpenApiParameter, _SerializerType]]] = None,
@@ -209,12 +259,15 @@ def extend_schema(
         summary: Optional[str] = None,
         deprecated: Optional[bool] = None,
         tags: Optional[List[str]] = None,
+        filters: Optional[bool] = None,
         exclude: bool = False,
         operation: Optional[Dict] = None,
         methods: Optional[List[str]] = None,
         versions: Optional[List[str]] = None,
         examples: Optional[List[OpenApiExample]] = None,
         extensions: Optional[Dict[str, Any]] = None,
+        callbacks: Optional[List[OpenApiCallback]] = None,
+        external_docs: Optional[Union[Dict[str, str], str]] = None,
 ) -> Callable[[F], F]:
     """
     Decorator mainly for the "view" method kind. Partially or completely overrides
@@ -254,6 +307,7 @@ def extend_schema(
     :param summary: an optional short summary of the description
     :param deprecated: mark operation as deprecated
     :param tags: override default list of tags
+    :param filters: ignore list detection and forcefully enable/disable filter discovery
     :param exclude: set True to exclude operation from schema
     :param operation: manually override what auto-discovery would generate. you must
         provide a OpenAPI3-compliant dictionary that gets directly translated to YAML.
@@ -261,6 +315,10 @@ def extend_schema(
     :param versions: scope extend_schema to specific API version. matches all by default.
     :param examples: attach request/response examples to the operation
     :param extensions: specification extensions, e.g. ``x-badges``, ``x-code-samples``, etc.
+    :param callbacks: associate callbacks with this endpoint
+    :param external_docs: Link external documentation. Provide a dict with an "url" key and
+        optionally a "description" key. For convenience, if only a string is given it is
+        treated as the URL.
     :return:
     """
     if methods is not None:
@@ -311,7 +369,7 @@ def extend_schema(
                 return super().get_override_parameters()
 
             def get_auth(self):
-                if auth and is_in_scope(self):
+                if auth is not None and is_in_scope(self):
                     return auth
                 return super().get_auth()
 
@@ -355,12 +413,28 @@ def extend_schema(
                     return extensions
                 return super().get_extensions()
 
+            def get_filter_backends(self):
+                if filters is not None and is_in_scope(self):
+                    return getattr(self.view, 'filter_backends', []) if filters else []
+                return super().get_filter_backends()
+
+            def get_callbacks(self):
+                if callbacks is not None and is_in_scope(self):
+                    return callbacks
+                return super().get_callbacks()
+
+            def get_external_docs(self):
+                if external_docs is not None and is_in_scope(self):
+                    return external_docs
+                return super().get_external_docs()
+
         if inspect.isclass(f):
             # either direct decoration of views, or unpacked @api_view from OpenApiViewExtension
             if operation_id is not None or operation is not None:
                 error(
                     f'using @extend_schema on viewset class {f.__name__} with parameters '
-                    f'operation_id or operation will most likely result in a broken schema.'
+                    f'operation_id or operation will most likely result in a broken schema.',
+                    delayed=f,
                 )
             # reorder schema class MRO so that view method annotation takes precedence
             # over view class annotation. only relevant if there is a method annotation
@@ -488,14 +562,19 @@ def extend_schema_view(**kwargs) -> Callable[[F], F]:
             if method_name not in available_view_methods:
                 warn(
                     f'@extend_schema_view argument "{method_name}" was not found on view '
-                    f'{view.__name__}. method override for "{method_name}" will be ignored.'
+                    f'{view.__name__}. method override for "{method_name}" will be ignored.',
+                    delayed=view
                 )
                 continue
 
             # the context of derived methods must not be altered, as it belongs to the
             # other view. create a new context so the schema can be safely stored in the
             # wrapped_method. view methods that are not derived can be safely altered.
-            method_decorator(isolate_view_method(view, method_name))
+            if hasattr(method_decorator, '__iter__'):
+                for sub_method_decorator in method_decorator:
+                    sub_method_decorator(isolate_view_method(view, method_name))
+            else:
+                method_decorator(isolate_view_method(view, method_name))
         return view
 
     return decorator
