@@ -1,5 +1,6 @@
 import json
 from collections import namedtuple
+from importlib import import_module
 from typing import Any, Dict
 
 from django.conf import settings
@@ -15,7 +16,7 @@ from drf_spectacular.plumbing import get_relative_url, set_query_parameters
 from drf_spectacular.renderers import (
     OpenApiJsonRenderer, OpenApiJsonRenderer2, OpenApiYamlRenderer, OpenApiYamlRenderer2,
 )
-from drf_spectacular.settings import spectacular_settings
+from drf_spectacular.settings import patched_settings, spectacular_settings
 from drf_spectacular.types import OpenApiTypes
 from drf_spectacular.utils import OpenApiParameter, extend_schema
 
@@ -53,25 +54,48 @@ class SpectacularAPIView(APIView):
     serve_public = spectacular_settings.SERVE_PUBLIC
     urlconf = spectacular_settings.SERVE_URLCONF
     api_version = None
+    custom_settings = None
+    patterns = None
 
     @extend_schema(**SCHEMA_KWARGS)
     def get(self, request, *args, **kwargs):
+        # special handling of custom urlconf parameter
         if isinstance(self.urlconf, list) or isinstance(self.urlconf, tuple):
             ModuleWrapper = namedtuple('ModuleWrapper', ['urlpatterns'])
-            self.urlconf = ModuleWrapper(tuple(self.urlconf))
+            if all(isinstance(i, str) for i in self.urlconf):
+                # list of import string for urlconf
+                patterns = []
+                for item in self.urlconf:
+                    url = import_module(item)
+                    patterns += url.urlpatterns
+                self.urlconf = ModuleWrapper(tuple(patterns))
+            else:
+                # explicitly resolved urlconf
+                self.urlconf = ModuleWrapper(tuple(self.urlconf))
 
-        if settings.USE_I18N and request.GET.get('lang'):
-            with translation.override(request.GET.get('lang')):
+        with patched_settings(self.custom_settings):
+            if settings.USE_I18N and request.GET.get('lang'):
+                with translation.override(request.GET.get('lang')):
+                    return self._get_schema_response(request)
+            else:
                 return self._get_schema_response(request)
-        else:
-            return self._get_schema_response(request)
 
     def _get_schema_response(self, request):
         # version specified as parameter to the view always takes precedence. after
         # that we try to source version through the schema view's own versioning_class.
         version = self.api_version or request.version or self._get_version_parameter(request)
-        generator = self.generator_class(urlconf=self.urlconf, api_version=version)
-        return Response(generator.get_schema(request=request, public=self.serve_public))
+        generator = self.generator_class(urlconf=self.urlconf, api_version=version, patterns=self.patterns)
+        return Response(
+            data=generator.get_schema(request=request, public=self.serve_public),
+            headers={"Content-Disposition": f'inline; filename="{self._get_filename(request, version)}"'}
+        )
+
+    def _get_filename(self, request, version):
+        return "{title}{version}.{suffix}".format(
+            title=spectacular_settings.TITLE or 'schema',
+            version=f' ({version})' if version else '',
+            suffix=self.perform_content_negotiation(request, force=True)[0].format
+        )
 
     def _get_version_parameter(self, request):
         version = request.GET.get('version')
@@ -104,25 +128,47 @@ class SpectacularSwaggerView(APIView):
 
     @extend_schema(exclude=True)
     def get(self, request, *args, **kwargs):
-        schema_url = self.url or get_relative_url(reverse(self.url_name, request=request))
         return Response(
             data={
                 'title': self.title,
                 'dist': self._swagger_ui_dist(),
                 'favicon_href': self._swagger_ui_favicon(),
-                'schema_url': set_query_parameters(
-                    url=schema_url,
-                    lang=request.GET.get('lang')
-                ),
+                'schema_url': self._get_schema_url(request),
                 'settings': self._dump(spectacular_settings.SWAGGER_UI_SETTINGS),
                 'oauth2_config': self._dump(spectacular_settings.SWAGGER_UI_OAUTH2_CONFIG),
-                'template_name_js': self.template_name_js
+                'template_name_js': self.template_name_js,
+                'csrf_header_name': self._get_csrf_header_name(),
+                'schema_auth_names': self._dump(self._get_schema_auth_names()),
             },
             template_name=self.template_name,
         )
 
     def _dump(self, data):
-        return data if isinstance(data, str) else json.dumps(data)
+        return data if isinstance(data, str) else json.dumps(data, indent=2)
+
+    def _get_schema_url(self, request):
+        schema_url = self.url or get_relative_url(reverse(self.url_name, request=request))
+        return set_query_parameters(
+            url=schema_url,
+            lang=request.GET.get('lang'),
+            version=request.GET.get('version')
+        )
+
+    def _get_csrf_header_name(self):
+        csrf_header_name = settings.CSRF_HEADER_NAME
+        if csrf_header_name.startswith('HTTP_'):
+            csrf_header_name = csrf_header_name[5:]
+        return csrf_header_name.replace('_', '-')
+
+    def _get_schema_auth_names(self):
+        from drf_spectacular.extensions import OpenApiAuthenticationExtension
+        if spectacular_settings.SERVE_PUBLIC:
+            return []
+        auth_extensions = [
+            OpenApiAuthenticationExtension.get_match(klass)
+            for klass in self.authentication_classes
+        ]
+        return [auth.name for auth in auth_extensions if auth]
 
     def _swagger_ui_dist(self):
         if spectacular_settings.SWAGGER_UI_DIST == 'SIDECAR':
@@ -145,15 +191,13 @@ class SpectacularSwaggerSplitView(SpectacularSwaggerView):
     @extend_schema(exclude=True)
     def get(self, request, *args, **kwargs):
         if request.GET.get('script') is not None:
-            schema_url = self.url or get_relative_url(reverse(self.url_name, request=request))
             return Response(
                 data={
-                    'schema_url': set_query_parameters(
-                        url=schema_url,
-                        lang=request.GET.get('lang')
-                    ),
+                    'schema_url': self._get_schema_url(request),
                     'settings': self._dump(spectacular_settings.SWAGGER_UI_SETTINGS),
                     'oauth2_config': self._dump(spectacular_settings.SWAGGER_UI_OAUTH2_CONFIG),
+                    'csrf_header_name': self._get_csrf_header_name(),
+                    'schema_auth_names': self._dump(self._get_schema_auth_names()),
                 },
                 template_name=self.template_name_js,
                 content_type='application/javascript',
@@ -186,13 +230,12 @@ class SpectacularRedocView(APIView):
 
     @extend_schema(exclude=True)
     def get(self, request, *args, **kwargs):
-        schema_url = self.url or get_relative_url(reverse(self.url_name, request=request))
-        schema_url = set_query_parameters(schema_url, lang=request.GET.get('lang'))
+
         return Response(
             data={
                 'title': self.title,
                 'dist': self._redoc_dist(),
-                'schema_url': schema_url,
+                'schema_url': self._get_schema_url(request),
             },
             template_name=self.template_name
         )
@@ -201,3 +244,11 @@ class SpectacularRedocView(APIView):
         if spectacular_settings.REDOC_DIST == 'SIDECAR':
             return _get_sidecar_url('redoc')
         return spectacular_settings.REDOC_DIST
+
+    def _get_schema_url(self, request):
+        schema_url = self.url or get_relative_url(reverse(self.url_name, request=request))
+        return set_query_parameters(
+            url=schema_url,
+            lang=request.GET.get('lang'),
+            version=request.GET.get('version')
+        )
