@@ -10,11 +10,12 @@ from rest_framework.test import APIClient
 from drf_spectacular.types import OpenApiTypes
 from drf_spectacular.utils import OpenApiExample, extend_schema, extend_schema_field
 from tests import assert_schema, generate_schema
+from tests.models import SimpleModel, SimpleSerializer
 
 try:
     from django_filters.rest_framework import (
-        AllValuesFilter, BaseInFilter, BooleanFilter, CharFilter, DjangoFilterBackend, FilterSet,
-        ModelChoiceFilter, ModelMultipleChoiceFilter, MultipleChoiceFilter, NumberFilter,
+        AllValuesFilter, BaseInFilter, BooleanFilter, CharFilter, ChoiceFilter, DjangoFilterBackend,
+        FilterSet, ModelChoiceFilter, ModelMultipleChoiceFilter, MultipleChoiceFilter, NumberFilter,
         NumericRangeFilter, OrderingFilter, RangeFilter, UUIDFilter,
     )
 except ImportError:
@@ -29,6 +30,7 @@ except ImportError:
             pass
 
     CharFilter = NumberFilter
+    ChoiceFilter = NumberFilter
     OrderingFilter = NumberFilter
     BaseInFilter = NumberFilter
     BooleanFilter = NumberFilter
@@ -87,6 +89,7 @@ class ProductFilter(FilterSet):
     int_id = NumberFilter(method='filter_method_typed')
     number_id = NumberFilter(method='filter_method_untyped', help_text='some injected help text')
     number_id_ext = NumberFilter(method=external_filter_method)
+    email = CharFilter(method='filter_method_decorated')
     # implicit filter declaration
     subproduct__sub_price = NumberFilter()  # reverse relation
     other_sub_product__uuid = UUIDFilter()  # forward relation
@@ -115,6 +118,17 @@ class ProductFilter(FilterSet):
         RangeFilter(field_name='price_vat')
     )
 
+    def get_choices(*args, **kwargs):
+        return (('A', 'aaa'),)
+
+    cat_callable = ChoiceFilter(field_name="category", choices=get_choices)
+
+    # will guess type from choices as a last resort
+    untyped_choice_field_method_with_explicit_choices = ChoiceFilter(
+        method="filter_method_untyped",
+        choices=[(1, 'one')],
+    )
+
     class Meta:
         model = Product
         fields = [
@@ -128,6 +142,11 @@ class ProductFilter(FilterSet):
     def filter_method_untyped(self, queryset, name, value):
         return queryset.filter(id=int(value))  # pragma: no cover
 
+    # email makes no sense here. it's just to test decoration
+    @extend_schema_field(OpenApiTypes.EMAIL)
+    def filter_method_decorated(self, queryset, name, value):
+        return queryset.filter(id=int(value))  # pragma: no cover
+
 
 @extend_schema(
     examples=[
@@ -211,8 +230,12 @@ def test_django_filters_requests(no_warnings):
     response = APIClient().get('/api/products/?multi_cat=A&multi_cat=B')
     assert response.status_code == 200, response.content
     assert len(response.json()) == 1
+    response = APIClient().get('/api/products/?cat_callable=A')
+    assert response.status_code == 200, response.content
+    assert len(response.json()) == 1
 
 
+@pytest.mark.contrib('django_filter')
 def test_through_model_multi_choice_filter(no_warnings):
     class RelationModel(models.Model):
         test = models.CharField(max_length=50)
@@ -246,11 +269,10 @@ def test_through_model_multi_choice_filter(no_warnings):
     generate_schema('x', TestViewSet)
 
 
+@pytest.mark.contrib('django_filter')
 def test_boolean_filter_subclassing_in_different_import_path(no_warnings):
     # this import is important as there is a override via subclassing
     # happening in django_filter.rest_framework.filters
-    import django_filters
-
     class DjangoFilterDummyModel(models.Model):
         seen = models.DateTimeField(null=True)
 
@@ -264,7 +286,7 @@ def test_boolean_filter_subclassing_in_different_import_path(no_warnings):
             model = DjangoFilterDummyModel
             fields = []
 
-        seen = django_filters.BooleanFilter(field_name="seen", lookup_expr="isnull")
+        seen = BooleanFilter(field_name="seen", lookup_expr="isnull")
 
     class XViewSet(viewsets.ModelViewSet):
         queryset = DjangoFilterDummyModel.objects.all()
@@ -276,3 +298,39 @@ def test_boolean_filter_subclassing_in_different_import_path(no_warnings):
     assert schema['paths']['/x/']['get']['parameters'] == [
         {'in': 'query', 'name': 'seen', 'schema': {'type': 'boolean'}}
     ]
+
+
+@pytest.mark.contrib('django_filter')
+def test_filters_on_retrieve_operations(no_warnings):
+    from django_filters import FilterSet
+    from django_filters.rest_framework import DjangoFilterBackend
+
+    class SimpleFilterSet(FilterSet):
+        pages = BaseInFilter(field_name='id')
+
+        class Meta:
+            model = SimpleModel
+            fields = '__all__'
+
+    class XViewset(viewsets.GenericViewSet):
+        queryset = SimpleModel.objects.all()
+        serializer_class = SimpleSerializer
+        filterset_class = SimpleFilterSet
+        filter_backends = [DjangoFilterBackend]
+
+        @extend_schema(
+            responses={(200, 'application/pdf'): OpenApiTypes.BINARY},
+            filters=True,
+        )
+        def retrieve(self, request, *args, **kwargs):
+            pass  # pragma: no cover
+
+    schema = generate_schema('/x', XViewset)
+    assert schema['paths']['/x/{id}/']['get']['parameters'][1] == {
+        'in': 'query',
+        'name': 'pages',
+        'schema': {'type': 'array', 'items': {'type': 'integer'}},
+        'description': 'Multiple values may be separated by commas.',
+        'explode': False,
+        'style': 'form'
+    }
